<?php

require 'classes/AddReward.php';

class SignController{

    protected function getMysql($account_id)
    {
        $mysql_conf = getMysqlConfig(crc32($account_id));
        $conn = new phpcommon\Mysql(array(
            'host' => $mysql_conf['host'],
            'port' => $mysql_conf['port'],
            'user' => $mysql_conf['user'],
            'passwd' => $mysql_conf['passwd'],
            'dbname' => 'gamedb2003_' . $mysql_conf['instance_id']
        ));
        return $conn;
    }

    protected function getSign($sign_id)
    {
        $g_conf_sign_cluster = require('../res/signDaily@signDaily.php');
        $sign_conf = getSignConfig($g_conf_sign_cluster, $sign_id);
        $s = array(
            'sign_id' => $sign_conf['sign_id'],
            'condition' => $sign_conf['condition'],
            'item_id' => $sign_conf['item_id'],
            'num' => $sign_conf['num'],
        );
        return $s;
    }

    protected function getExplode($string)
    {
        $delim = "|";
        $drop_multiply = explode($delim, $string);
        $delim1 = ":";
        $arr = array();
        for ($i = 0; $i < count($drop_multiply); $i++) {
            $mul = explode($delim1, $drop_multiply[$i]);
            array_push($arr, $mul);
        }
        return $arr;
    }

    protected function getParameter($para_id)
    {
        $g_conf_para_cluster = require('../res/parameter@parameter.php');
        $para_conf = getParameterConfig($g_conf_para_cluster, $para_id);
        $p = array(
            'id' => $para_conf['id'],
            'param_name' => $para_conf['param_name'],
            'param_value' => $para_conf['param_value'],
        );
        return $p;
    }

    protected function insertSign($account_id, $sign_days)
    {
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            die();
        }
        $ret = $conn->execScript('INSERT INTO sign(accountid, sign_id, signable, sign_time, create_time, modify_time) ' .
                                 ' VALUES(:accountid, :sign_id, 0, :sign_time, :create_time, :modify_time) ' .
                                 ' ON DUPLICATE KEY UPDATE accountid=:accountid, sign_id=:sign_id, signable=0, sign_time=:sign_time, modify_time=:modify_time;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':sign_id' => $sign_days,
                                     ':sign_time' => time(),
                                     ':create_time' => time(),
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
        }
    }

    public function signInfo()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        $last_sign_id = 0;
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }

        $rows = $conn->execQuery('SELECT sign_id, sign_time FROM sign WHERE accountid=:accountid;',
                                 array(
                                     ':accountid' => $account_id
                                 ));
        $nowTime = phpcommon\getdayseconds(time());
        if (!$rows) {
            $last_sign_id = 1;
            //插入签到列表
            $this->insertSign($account_id, $last_sign_id);
            //更新签到总天数
            $this->updateSignSum($account_id, $last_sign_id);
        } else {
            $last_sign_time = 0;
            foreach ($rows as $row) {
                if ($row['sign_id'] > $last_sign_id) {
                    $last_sign_time = $row['sign_time'];
                    $last_sign_id = $row['sign_id'];
                }
            }

            if ($nowTime - phpcommon\getdayseconds($last_sign_time) > 0) {
                //每日刷新
                $passed_days = floor(($nowTime - phpcommon\getdayseconds($last_sign_time)) / (3600 * 24));
                if ($passed_days > 7 - $last_sign_id) {
                    //跨周时删除老数据
                    $num = 0;
                    $sum = 0;
                    $ret = $conn->execScript('DELETE from sign WHERE accountid=:accountid;',
                                             array(
                                                 ':accountid' => $account_id,
                                             ));
                    if (!$ret) {
                        die();
                        return;
                    }
                    if ($passed_days > 7) {
                        $num = ($passed_days + $last_sign_id) % 7;
                    } else {
                        $num = $last_sign_id + $passed_days - 7;
                    }
                    if ($num == 0) {
                        $sum = 7;
                    } else {
                        $sum = $num;
                    }
                    for ($i = 1; $i < $sum + 1; $i++) {
                        //插入补签列表
                        $this->insertSign($account_id, $i);
                    }
                } else {
                    for ($i = $last_sign_id + 1; $i < $passed_days + $last_sign_id + 1; $i++) {
                        //插入补签列表
                        $this->insertSign($account_id, $i);
                    }
                }
                //更新签到总天数
                $this->updateSignSum($account_id, $passed_days);
            }
        }

        $sign_list = array();
        $rows = $conn->execQuery('SELECT sign_id, signable FROM sign WHERE accountid=:accountid;',
                                 array(
                                     ':accountid' => $account_id
                                 ));
        foreach ($rows as $row) {
            array_push($sign_list, array(
                'sign_id' => $row['sign_id'],
                'signable' => $row['signable']
            ));
        }
        $rowUser = $conn->execQueryOne('SELECT sign_sum FROM user WHERE accountid=:accountid;',
                                       array(
                                           ':accountid' => $account_id
                                       ));
        //判断当前第几周
        $item_list = array();
        $week = ceil($rowUser['sign_sum'] / 7);
        $dayid = ($week - 1) * 7;
        //如果大于配置表最后一周，按最后一周奖励
        $g_conf_sign_cluster = require('../res/signDaily@signDaily.php');
        if ($dayid > count($g_conf_sign_cluster)) {
            $dayid = count($g_conf_sign_cluster) - 7;
        }
        for ($day = $dayid + 1; $day <= $dayid + 7; $day++) {
            $s = $this->getSign($day + 90000);
            array_push($item_list, array(
                'item_id' => $s['item_id'],
                'item_num' => $s['num'],
            ));
        }
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'sign_days' => $rowUser['sign_sum'],
            'sign_list' => $sign_list,
            'item_list' => $item_list
        ));
    }


    protected function updateSignSum($account_id, $sign_num)
    {
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            die();
        }
        $row = $conn->execQueryOne('SELECT sign_sum FROM user WHERE accountid=:accountid;',
                                   array(
                                       ':accountid' => $account_id
                                   ));
        $ret = $conn->execScript('UPDATE user SET sign_sum=:sign_sum, modify_time=:modify_time ' .
                                 ' WHERE accountid=:accountid;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':sign_sum' => $row['sign_sum'] + $sign_num,
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
        }
    }

    public function signReward()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        $row = $conn->execQueryOne('SELECT signable FROM sign WHERE accountid=:accountid AND sign_id=:sign_id;',
                                   array(
                                       ':accountid' => $account_id,
                                       ':sign_id' => $_REQUEST['sign_id'],
                                   ));
        if (!$row || $row['signable'] == 1) {
            phpcommon\sendError(ERR_USER_BASE + 2, '已签到');
            return;
        }
        $ret = $conn->execScript('UPDATE sign SET sign_time=:sign_time, signable=1, modify_time=:modify_time ' .
                                 ' WHERE accountid=:accountid AND sign_id=:sign_id;',
                                 array(
                                     ':accountid' => $account_id,
                                     ':sign_id' => $_REQUEST['sign_id'],
                                     ':sign_time' => time(),
                                     ':modify_time' => time()
                                 ));
        if (!$ret) {
            die();
            return;
        }
        $rowUser = $conn->execQueryOne('SELECT sign_sum FROM user WHERE accountid=:accountid;',
                                       array(
                                           ':accountid' => $account_id
                                       ));
        //获得奖励
        //判断当前第几周
        $item_list = array();
        $week = ceil($rowUser['sign_sum'] / 7);
        $dayid = ($week - 1) * 7 + $_REQUEST['sign_id'];
        //如果大于配置表最后一周，按最后一周奖励
        $g_conf_sign_cluster = require('../res/signDaily@signDaily.php');
        if ($dayid > count($g_conf_sign_cluster)) {
            $dayid = count($g_conf_sign_cluster) - 7;
        }
        $s = $this->getSign($dayid + 90000);
        $item_id = $s['item_id'];
        $num = $s['num'];
        array_push($item_list, array(
            'item_id' => $s['item_id'],
            'item_num' => $s['num'],
        ));
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
            'item_list' => $item_list
        ));
    }

    public function signDoubleReward()
    {
        $account_id = $_REQUEST['account_id'];
        //登录校验
        $login = loginVerify($account_id, $_REQUEST['session_id']);
        if (!$login) {
            phpcommon\sendError(ERR_USER_BASE + 1, 'session无效');
            return;
        }
        $conn = $this->getMysql($account_id);
        if (!$conn) {
            phpcommon\sendError(ERR_USER_BASE + 1, '没有这个玩家');
            return;
        }
        $rowUser = $conn->execQueryOne('SELECT sign_sum FROM user WHERE accountid=:accountid;',
                                       array(
                                           ':accountid' => $account_id
                                       ));
        //获得奖励
        //判断当前第几周
        $item_list = array();
        $week = ceil($rowUser['sign_sum'] / 7);
        $dayid = ($week - 1) * 7 + $_REQUEST['sign_id'];

        //如果大于配置表最后一周，按最后一周奖励
        $g_conf_sign_cluster = require('../res/signDaily@signDaily.php');
        if ($dayid > count($g_conf_sign_cluster)) {
            $dayid = count($g_conf_sign_cluster) - 7;
        }
        $s = $this->getSign($dayid + 90000);
        $item_id = $s['item_id'];
        $num = $s['num'];
        $p = $this->getParameter(SIGNREWARD_TIMES);
        $times = $p['param_value'] - 1;
        $num = $num * $times;
        echo json_encode(array(
            'errcode' => 0,
            'errmsg' => '',
        ));
    }

}
?>
